package cluster

import (
	"context"
	"encoding/json"
	"net/http"

	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/constants"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/validators"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/cluster/models"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/cluster/requests"
)

// List gets a list of all clusters
func (c Cluster) List(ctx context.Context, req requests.ListClusterRequest) (*models.ListClusterResponse, error) {

	res, err := c.ListRaw(ctx, req)
	if err != nil {
		return nil, err
	}

	list := models.ListClusterResponse{}

	err = json.Unmarshal(res, &list)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// ListRaw gets a list of all clusters as an array of bytes
func (c Cluster) ListRaw(ctx context.Context, req requests.ListClusterRequest) ([]byte, error) {

	if err := validators.ValidateRequest(req); err != nil {
		return nil, validators.ValidationErrors(validators.GetErrors(err))
	}

	url := constants.APIv0 + "/cluster"

	res, err := c.client.ApiCall(ctx, http.MethodGet, url, req)
	return res, err
}
