package cluster

import (
	"context"
	"encoding/json"
	"fmt"
	"net/http"

	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/constants"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/validators"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/cluster/models"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/cluster/requests"
)

// ListVNetworks Returns a list of v_networks for a specific cluster
func (c Cluster) ListVNetworks(ctx context.Context, req requests.ListVNetworksRequest) (*models.ListVNetwork, error) {

	res, err := c.ListVNetworksRaw(ctx, req)
	if err != nil {
		return nil, err
	}

	list := models.ListVNetwork{}

	err = json.Unmarshal(res, &list)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// ListVNetworksRaw gets a list of all v_network the user has access to as an array of bytes
func (c Cluster) ListVNetworksRaw(ctx context.Context, req requests.ListVNetworksRequest) ([]byte, error) {

	if err := validators.ValidateRequest(req); err != nil {
		return nil, validators.ValidationErrors(validators.GetErrors(err))
	}

	url := constants.APIv1 + "/cluster/" + fmt.Sprint(req.ClusterID) + "/v_network"

	res, err := c.client.ApiCall(ctx, http.MethodGet, url, req)
	return res, err
}
