package respool

import (
	"context"
	"encoding/json"
	"net/http"

	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/constants"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/validators"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/respool/models"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/respool/requests"
)

// List gets a list of all resource pools
func (rp ResourcePool) List(ctx context.Context, req requests.ListResPoolRequest) (*models.ListResPoolResponse, error) {

	res, err := rp.ListRaw(ctx, req)
	if err != nil {
		return nil, err
	}

	list := models.ListResPoolResponse{}

	err = json.Unmarshal(res, &list)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// ListRaw gets a list of all resource pools as an array of bytes
func (rp ResourcePool) ListRaw(ctx context.Context, req requests.ListResPoolRequest) ([]byte, error) {

	if err := validators.ValidateRequest(req); err != nil {
		return nil, validators.ValidationErrors(validators.GetErrors(err))
	}

	url := constants.APIv0 + "/resource_pool"

	res, err := rp.client.ApiCall(ctx, http.MethodGet, url, req)
	return res, err
}
