package vm

import (
	"context"
	"encoding/json"
	"fmt"
	"net/http"

	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/constants"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/internal/validators"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/vm/models"
	"repository.basistech.ru/BASIS/dynamix-standard-go-sdk/pkg/vm/requests"
)

// GetDisks return information about disks of the specified VM
func (vm VM) GetDisks(ctx context.Context, req requests.GetDisksRequest) (*models.ListDiskResponse, error) {

	res, err := vm.GetDisksRaw(ctx, req)
	if err != nil {
		return nil, err
	}

	list := models.ListDiskResponse{}

	err = json.Unmarshal(res, &list)
	if err != nil {
		return nil, err
	}
	return &list, nil
}

// GetDisksRaw return information about disks of the specified VM an array of bytes
func (vm VM) GetDisksRaw(ctx context.Context, req requests.GetDisksRequest) ([]byte, error) {

	if err := validators.ValidateRequest(req); err != nil {
		return nil, validators.ValidationErrors(validators.GetErrors(err))
	}

	url := constants.APIv0 + "/vm/" + fmt.Sprint(req.VmID) + "/device/hard_disk"

	res, err := vm.client.ApiCall(ctx, http.MethodGet, url, req)
	return res, err
}
